use {
	async_sqlite :: {
		* ,
		Error :: * ,
		rusqlite :: { OpenFlags , Error :: * , ffi :: ErrorCode :: * } ,
	} ,
	fallible_iterator :: * ,
	std :: path :: * ,
} ;

use crate :: { collection :: * , error :: { * , ZoodexError :: * } } ;



pub async fn get_collection_from_file () -> Result <Collection> {
	let sqlite_client = ClientBuilder :: new ()
		. path ("zoodex.sqlite")
		. flags ( OpenFlags :: SQLITE_OPEN_READ_ONLY | OpenFlags :: SQLITE_OPEN_NO_MUTEX )
		. open ()
		. await ? ;

	let collection = sqlite_client . conn ( |sqlite_connection| {
		let films = sqlite_connection
			. prepare ("select * from films order by release_date desc") ?
			. query (()) ?
			. map ( |row| {
				let uuid = row . get (0) ? ;
				let name = row . get (1) ? ;
				let original_name = row . get (2) ? ;
				let release_date = row . get (3) ? ;
				let runtime_minutes = row . get (4) ? ;
				let poster_file_path = row . get :: < _ , Option <String> > (5) ?
					. map ( PathBuf :: from ) ;
				let video_file_path = row . get :: < _ , Option <String> > (6) ?
					. map ( PathBuf :: from ) ;

				Ok ( Film {
					uuid ,
					name ,
					original_name ,
					release_date ,
					runtime_minutes ,
					poster_file_path ,
					video_file_path ,
				} )
			} )
			. collect () ? ;

		let series = sqlite_connection
			. prepare ("select * from series") ?
			. query (()) ?
			. map ( |row| {
				let uuid = row . get (0) ? ;
				let name = row . get (1) ? ;
				let original_name = row . get (2) ? ;
				let poster_file_path = row. get :: < _ , Option <String> > (3) ?
					. map ( PathBuf :: from ) ;

				Ok ( Series { uuid , name , original_name , poster_file_path } )
			} )
			. collect () ? ;

		Ok ( Collection { films , series } )
	} ) . await ? ;

	Ok (collection)
}

impl From <Error> for ZoodexError {
	fn from ( error : Error ) -> Self {
		match error {
			Rusqlite (error) => ZoodexError :: from (error) ,
			_ => panic ! () ,
		}
	}
}

impl From < rusqlite :: Error > for ZoodexError {
	fn from ( error : rusqlite :: Error ) -> Self {
		match error {
			SqliteFailure ( error , _ ) => {
				match error . code {
					CannotOpen => CollectionFileReadError ,
					_ => panic ! () ,
				}
			} ,
			_ => panic ! () ,
		}
	}
}
